"use strict";
const PLUGIN_NAME = "gulp-gwd";
const through = require("through2");
const gutil = require("gulp-util");
const PluginError = gutil.PluginError;
const fs = require("fs");
const path = require("path");

//const img_regex = /<\s*img\s*.[^>]*? (?:src|source)\s*=\s*['"]?([^'"\s]+)['"]?/gi;
//const css_regex = /url\s*\(['"]?(.[^'"\s]+)['"]?\)/gi;
//const script_regex = /<\s*script\s*[^>]*? src\s*=\s*['"]?([^'"\s]+)['"]?/gi;
//const link_regex = /<\s*link\s*.[^>]*? href\s*=\s*['"]?([^'"\s]+)['"]?/gi;

const ignoreExts = ["", ".cmd", ".db", ".psd"];
const ignoreFiles = ["gwd_workspace.json"];

var assets = [];
var checkedAssets = [];
var folder = "";

module.exports = function(options) {
	var indexExists = false;
	var htmlFiles = [];
	var removedFiles = [];
	
	options = options || {};
	options.step = options.step || "";

	function filter(file, encoding, callback) {
		if (file.isNull()) { callback(null, file); return; }
		if (file.isStream()) { callback(new PluginError(PLUGIN_NAME, "Streaming not supported")); return; }
		try {
			var filepath = file.history;
			if (~ignoreExts.indexOf(path.extname(filepath)) || ~ignoreFiles.indexOf(path.basename(filepath))){ callback(); return; } 
			this.push(file);
		} catch (err) {
			this.emit("error", new PluginError(PLUGIN_NAME, err));
		}

		callback();
	}
	function getFiles(file, encoding, callback) {
		if (file.isNull()) { callback(null, file); return; }
		if (file.isStream()) { callback(new PluginError(PLUGIN_NAME, "Streaming not supported")); return; }
		try {
			var filepath = file.history;
			var extname = path.extname(filepath);
			var filename = path.basename(filepath);
			
			if (extname === ".html" && filename !== "index.html") { 
				htmlFiles.push(String(filepath));
			} else {
				assets.push(String(filepath));
				if (filename === "index.html"){
					checkedAssets.push(String(filepath));
					indexExists = true;
				} 
			}
			this.push(file);
			
		} catch (err) {
			this.emit("error", new PluginError(PLUGIN_NAME, err));
		}
		callback();
	}
	function renameHtml(callback) {
		if (indexExists === true) { callback(); return }
		if (htmlFiles.length === 1){
			var oldPath = path.normalize(htmlFiles[0]);
			var newPath = path.join(path.dirname(oldPath), "index.html"); 
			fs.renameSync(oldPath, newPath);
		} else if (htmlFiles.length > 1){
			this.emit("error", new PluginError(PLUGIN_NAME, "Found " + htmlFiles.length + " .html files, just 1 needed"));
		} else {
			this.emit("error", new PluginError(PLUGIN_NAME, "HTML file is missing"));
		}
		callback();
	};
	
	function replacer(str, group, offset) {
		if (group.indexOf("data:") === 0 || path.isAbsolute(group) === true) return str;
		var newName = String(path.basename(group));
		
		if (group !== newName){
			var oldPath = path.join(folder, group);
			var newPath = path.join(folder, newName);
			
			if (!~checkedAssets.indexOf(newPath)){
				checkedAssets.push(newPath);
				if (fs.existsSync(oldPath) && fs.statSync(oldPath).isFile()) {
					fs.renameSync(oldPath, newPath);
				}
			}
			return str.replace(group, newName);
		}
		
		return str;
		
	}
	function index(file, encoding, callback) {
		if (file.isNull()) { callback(null, file); return; }
		if (file.isStream()) { callback(new PluginError(PLUGIN_NAME, "Streaming not supported")); return; }
		try {
			var filepath = file.history;
			var filename = path.basename(filepath);
			if (filename !== "index.html" || folder !== ""){ callback(); return; }
			folder = path.dirname(filepath);
			
			var regAssets = [];
			for (var i = 0; i < assets.length; i++) {
				regAssets[i] = path.relative(folder, assets[i]).replace(/(\\|\/)/, unescape('[\\/|\\\\]'));
			}
			var assetsRegExp = new RegExp(unescape("[\"\'\()][\\/|\\\\]?") + "(" + regAssets.join("|") + ")[\"\'\)]", "g");
			
			var data = file.contents.toString();
			data = data.replace(assetsRegExp, replacer);
			file.contents = new Buffer(data);
			this.push(file);
			
		} catch (err) {
			this.emit("error", new PluginError(PLUGIN_NAME, err));
		}
		callback();
	}
	
	function output_filter(file, encoding, callback) {
		if (file.isNull()) { callback(null, file); return; }
		if (file.isStream()) { callback(new PluginError(PLUGIN_NAME, "Streaming not supported")); return; }
		try {
			var filepath = file.history;
			var filename = path.basename(filepath);
			
			if (~checkedAssets.indexOf(String(filepath))){ 
				this.push(file);
			} else {
				var extname = path.extname(filename);
				if (extname !== ".js" && extname !== ".css") removedFiles.push(filename);
			}
		} catch (err) {
			this.emit("error", new PluginError(PLUGIN_NAME, err));
		}
		
		callback();
	}
	function output_report(callback) {
		if (removedFiles.length > 0){
			gutil.log(gutil.colors.bold.yellow("Removed files: " + removedFiles.join(", ")));
		}
		callback();
	};
	function emptyFolders(file, encoding, callback) {
		if (file.isNull()) { callback(null, file); return; }
		if (file.isStream()) { callback(new PluginError(PLUGIN_NAME, "Streaming not supported")); return; }
		try {
			var filepath = file.history;
			var filename = path.basename(filepath);
			if (filename !== "index.html"){ callback(); return; }
			
			var outputFolder = path.dirname(filepath);
			
			var all = fs.readdirSync(outputFolder);
			for (var i = 0; i < all.length; i++) {
				var elemPath = path.join(outputFolder, all[i]);
				if (fs.statSync(elemPath).isDirectory() === true){
					var elemFolder = fs.readdirSync(elemPath);
					if (elemFolder.length === 0){
						fs.rmdirSync(elemPath);
					}
				}
			}
			this.push(file);
		} catch (err) {
			this.emit("error", new PluginError(PLUGIN_NAME, err));
		}

		callback();
	}
	if (options.step === "filter"){
		return through.obj(filter);
	} else if (options.step === "init"){
		return through.obj(getFiles, renameHtml);
	} else if (options.step === "output"){
		return through.obj(output_filter, output_report);
	} else if (options.step === "emptyFolders"){
		return through.obj(emptyFolders);
	} else {
		return through.obj(index);
	}
};